"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stripBoolean = exports.flex = exports.breakpoints = exports.generateStyles = exports.drawDebug = undefined;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _styledComponents = require("styled-components");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var drawDebug = exports.drawDebug = function drawDebug() {
  return ["background: rgba(0, 255, 255, 0.1);", "border: 1px dashed rgba(255, 0, 255, 1);"];
};

/**
 * Returns % width based on alias. Returns
 * string with if alias is not passed
 * @param {string} alias
 */
var processAlias = function processAlias(alias) {
  switch (alias) {
    case "half":
      return "50%";
      break;
    case "quarter":
      return "25%";
      break;
    case "third":
      return "33.3333333%";
      break;
    case "twoThirds":
      return "66.666666%";
      break;
    case "threeQuarters":
      return "75%";
      break;
    default:
      return alias;
  }
};

var cssProperties = {
  padding: function padding(data) {
    return "padding: " + data + ";";
  },
  paddingLeft: function paddingLeft(data) {
    return "padding-left: " + data + ";";
  },
  paddingRight: function paddingRight(data) {
    return "padding-right: " + data + ";";
  },
  paddingTop: function paddingTop(data) {
    return "padding-top: " + data + ";";
  },
  paddingBottom: function paddingBottom(data) {
    return "padding-bottom: " + data + ";";
  },
  margin: function margin(data) {
    return "margin: " + data + ";";
  },
  marginLeft: function marginLeft(data) {
    return "margin-left: " + data + ";";
  },
  marginRight: function marginRight(data) {
    return "margin-right: " + data + ";";
  },
  marginTop: function marginTop(data) {
    return "margin-top: " + data + ";";
  },
  marginBottom: function marginBottom(data) {
    return "margin-bottom: " + data + ";";
  },
  width: function width(data) {
    return "width: " + processAlias(data) + ";";
  },
  height: function height(data) {
    return "height: " + processAlias(data) + ";";
  },
  visibility: function visibility(data) {
    return "visibility: " + data + ";";
  },
  display: function display(data) {
    return "display: " + data + ";";
  },
  opacity: function opacity(data) {
    return "opacity: " + data + ";";
  },
  color: function color(data) {
    return "color: " + data + ";";
  },
  background: function background(data) {
    return "background: " + data + ";";
  },
  border: function border(data) {
    return "border: " + data + ";";
  },
  fontSize: function fontSize(data) {
    return "font-size: " + data + ";";
  },
  fontWeight: function fontWeight(data) {
    return "font-weight: " + data + ";";
  },
  fontStyle: function fontStyle(data) {
    return "font-style: " + data + ";";
  },
  fontFamily: function fontFamily(data) {
    return "font-family: " + data + ";";
  },
  lineHeight: function lineHeight(data) {
    return "line-height: " + data + ";";
  },
  textTransform: function textTransform(data) {
    return "text-transform: " + data + ";";
  },
  hidden: function hidden() {
    return "display: none;";
  }
};

/**
 * Returns an array of valid css declarations generated
 * from the react props supplied.
 *
 * @param {array} props list of react props
 */
var generateStyles = exports.generateStyles = function generateStyles(props) {
  var cssProps = [];
  console.log(props);
  Object.keys(props).map(function (prop) {
    if (cssProperties[prop]) cssProps.push(cssProperties[prop](props[prop]));
  });
  console.log(cssProps);
  return cssProps;
};

/**
 * This will parse a breakpoint string (i.e. -500) and return
 * the operation needed to generate a media query
 * @param {string} breakpoint
 */
var parseBreakpoint = function parseBreakpoint(breakpoint) {
  switch (breakpoint.charAt(0)) {
    // less than
    case "-":
      return { lt: breakpoint.substring(1) };
      break;
    // more than
    case "+":
      return { gt: breakpoint.substring(1) };
      break;
    // range
    default:
      var range = breakpoint.split("-");
      if (range.length !== 2) {
        console.error("Invalid breakpoint range. The proper syntax for a range is: min-max");
      }
      return { range: range };
  }
};

/**
 * Generates a media query from a breakpoint operation
 *
 * @param {Object} operation
 */
var media = function media(operation) {
  if (operation.lt) {
    return "@media (max-width: " + operation.lt + "px)";
  } else if (operation.gt) {
    return "@media (min-width: " + operation.gt + "px)";
  } else if (operation.range) {
    return "@media (min-width: " + operation.range[0] + "px) and (max-width: " + operation.range[1] + "px)";
  }
};

/**
 * Generates valid CSS media queries from a component's props
 *
 * @param {Object} props
 */
var breakpoints = exports.breakpoints = function breakpoints(props) {
  if (!props.breakpoints) return null;

  return Object.keys(props.breakpoints).map(function (id) {
    return props[id] && [(0, _styledComponents.css)(["", "{", ";", ";}"], media(parseBreakpoint(props.breakpoints[id])), generateStyles(props[id]), flex(props[id]))];
  });
};

var translations = {
  direction: {
    horizontal: "row",
    vertical: "column"
  },
  wrap: {
    true: "wrap",
    1: "wrap",
    false: "nowrap",
    0: "nowrap"
  },
  align: {
    top: "flex-start",
    left: "flex-start",
    bottom: "flex-end",
    right: "flex-end",
    center: "center",
    middle: "center"
  }
};

var flex = exports.flex = function flex(_ref) {
  var flex = _ref.flex,
      fill = _ref.fill,
      direction = _ref.direction,
      wrap = _ref.wrap,
      valign = _ref.valign,
      halign = _ref.halign,
      shiftLeft = _ref.shiftLeft,
      shiftRight = _ref.shiftRight,
      shiftUp = _ref.shiftUp,
      shiftDown = _ref.shiftDown;

  var props = [];
  if (flex) props.push("flex: " + flex + ";");
  if (fill) props.push("flex: 1 1 auto;");
  if (direction) props.push("flex-direction: " + translations.direction[direction] + ";");
  if (wrap) props.push("flex-wrap: " + translations.wrap[wrap] + ";");

  if (halign) {
    if (direction === "horizontal") props.push("justify-content: " + translations.align[halign] + ";");else props.push("align-items: " + translations.align[halign] + ";");
  }

  if (valign) {
    if (direction === "horizontal") props.push("align-items: " + translations.align[valign] + ";");else props.push("justify-content: " + translations.align[valign] + ";");
  }

  if (shiftLeft) props.push("margin-right: auto;");
  if (shiftRight) props.push("margin-left: auto;");
  if (shiftUp) props.push("margin-bottom: auto;");
  if (shiftDown) props.push("margin-top: auto;");

  return props;
};

// eslint-disable-next-line max-len
var stripBoolean = function stripBoolean(Element) {
  return function (_ref2) {
    var debug = _ref2.debug,
        fluid = _ref2.fluid,
        fill = _ref2.fill,
        wrap = _ref2.wrap,
        props = _objectWithoutProperties(_ref2, ["debug", "fluid", "fill", "wrap"]);

    return _react2.default.createElement(Element, props);
  };
};
exports.stripBoolean = stripBoolean;